@@ -45,11 +45,11 @@ class Diagnose:
 
         # Find uuid rename keys using negative lookahead regex; text between { and "type": "rename"
         renameUUIDKeys = re.findall(
-            '\{(?:(?!\{|' + re.escape('"type": "rename"') + ')[\s\S])*' + re.escape('"type": "rename"'), jsonData)
+            r'\{(?:(?!\{|' + re.escape('"type": "rename"') + r')[\s\S])*' + re.escape('"type": "rename"'), jsonData)
         uuidRegex = re.compile(
-            '[0-9a-z]{9}\_[0-9a-z]{4}\_[0-9a-z]{4}\_[0-9a-z]{4}\_[0-9a-z]{12}')
+            r'[0-9a-z]{9}\_[0-9a-z]{4}\_[0-9a-z]{4}\_[0-9a-z]{4}\_[0-9a-z]{12}')
         uuidDollarRegex = re.compile(
-            '\$[0-9a-z]{9}\_[0-9a-z]{4}\_[0-9a-z]{4}\_[0-9a-z]{4}\_[0-9a-z]{12}')
+            r'\$[0-9a-z]{9}\_[0-9a-z]{4}\_[0-9a-z]{4}\_[0-9a-z]{4}\_[0-9a-z]{12}')
         renameUUIDKeys = list(set(re.findall(uuidRegex, str(renameUUIDKeys))))
 
         # Find all $uuid keys
@@ -128,11 +128,17 @@ class Diagnose:
 
         # Global settings
         resetCoverage = False
+        globalCheckCoverage = False
         globalAction = False
         globalVerbose = "to-console"
-        globalMapping = False   
 
-        runTimeStamp = time()
+        errorFile = None
+        # regex to match the string to-file in between quotes or double quotes
+        regex = re.compile(r'\"(to-file)\"|\'(to-file)\'')
+        if regex.search(str(rules)):
+            errorFile = open(str(floor(time())) + ".txt", "a")
+
+        globalMapping = False   
 
         # Contains "action" and "verbose"
         errorsFound = False
@@ -155,8 +161,7 @@ class Diagnose:
                 verbose = rules[j]["verbose"]
 
             if verbose == "to-file":
-                with open(str(runTimeStamp) + ".txt", "a") as errorFile:
-                    errorFile.write(str(rules[j]["column"]) + " mismatch row "+ str(i) + " - " + str(value) + "\n")
+                errorFile.write(str(rules[j]["column"]) + " mismatch row " + str(i) + " - " + str(value) + "\n")
 
             if verbose == "to-console":
                 print(str(rules[j]["column"]) + " mismatch row "+ str(i) + " - " + str(value))
@@ -169,6 +174,9 @@ class Diagnose:
 
             # Global rules
             if not("column" in rules[j]):
+                if "check-coverage" in rules[j]:
+                    globalCheckCoverage = rules[j]["check-coverage"]
+
                 if ("reset-coverage" in rules[j]):
                     if (bool)(rules[j]["reset-coverage"]):
                         resetCoverage = bool(rules[j]["reset-coverage"])
@@ -198,20 +206,20 @@ class Diagnose:
                 if "mapping" in rules[j]:
                     mapping = rules[j]["mapping"]
 
-                columnValues = columnValues.map(mapping)
+                columnValues = columnValues.replace(mapping)
 
             if "reset-coverage" in rules[j]:
                 if (bool)(rules[j]["reset-coverage"]):
                     resetCoverage = (bool(rules[j]["reset-coverage"]))
 
-            if "verbose" in rules[j]:
-                verbose = (rules[j]["verbose"])
-
-            if "check-coverage" in rules[j]:
+            if "check-coverage" in rules[j] or globalCheckCoverage != False:
                 if errorsFound and resetCoverage: errorsFound = False
                 else:
-                    if re.match(r"^[1-9][0-9]?$|^100$", rules[j]["check-coverage"]):
-                        columnValues = columnValues.sample(frac=int(rules[j]["check-coverage"]) / 100)
+                    checkCoverage = globalCheckCoverage
+                    if "check-coverage" in rules[j]:
+                        checkCoverage = (rules[j]["check-coverage"])
+                    if re.match(r"^[1-9][0-9]?$|^100$", checkCoverage):
+                        columnValues = columnValues.sample(frac=int(checkCoverage) / 100)
 
             if "selection" in rules[j]:
                 sortedSelection = sorted(rules[j]["selection"])
@@ -235,37 +243,43 @@ class Diagnose:
                 
                 if rules[j]["type"] == "boolean":
                     for i, value in columnValues.items():
-                        columnValues[i] = str(value).lower()
-                        if not re.match(r"^(true|false)$", str(value)):
+                        if value != True and value != False:
                             handleError(i, value, columnValues)
                 
                 if rules[j]["type"] == "float":
-                    if (dataframe['int'].dtype == float):
-                            continue
+                    if (columnValues.dtype == float):
+                        continue
 
                     for i, value in columnValues.items():
-                        if(type(value) != int):
+                        try:
+                            float(value)
+                        except ValueError:
                             handleError(i, value, columnValues)
                 
                 if rules[j]["type"] == "int":
-                    if (dataframe['int'].dtype == int):
+                    if (columnValues.dtype == int):
                             continue
 
                     for i, value in columnValues.items():
-                        if(type(value) != int):
+                        try:
+                            int(value)
+                        except ValueError:
                             handleError(i, value, columnValues)
-                        else:
-                            if (type(value) == float): handleError(i, value, columnValues)
-                            columnValues[i] = int(value)
 
                 if rules[j]["type"] == "positive-int":
                     for i, value in columnValues.items():
-                        if (type(value) == int and value >= 0 == False):
+                        try:
+                            if int(value) < 0:
+                                handleError(i, value, columnValues)
+                        except ValueError:
                             handleError(i, value, columnValues)
 
                 if rules[j]["type"] == "negative-int":
                     for i, value in columnValues.items():
-                        if (type(value) == int and value < 0 == False):
+                        try:
+                            if int(value) >= 0:
+                                handleError(i, value, columnValues)
+                        except ValueError:
                             handleError(i, value, columnValues)
 
                 if rules[j]["type"] == "letters":
@@ -274,7 +288,7 @@ class Diagnose:
                             handleError(i, value, columnValues)
 
                 if rules[j]["type"] == "postal_code":
-                    columnValues.str.replace(r"[^a-zA-Z0-9]+", "").str.upper()
+                    columnValues.str.replace(r"[^a-zA-Z0-9]+", "", regex=True).str.upper()
                     for i, value in columnValues.items():
 
                         if not re.match(r"^[1-9][0-9]{3}\s?[a-zA-Z]{2}$", str(value)):
@@ -296,5 +310,5 @@ class Diagnose:
                 continue
 
             dataframe[rules[j]["column"]].update(columnValues)
-            
+            errorsFound = False
         return dataframe
\ No newline at end of file
